

#include <qapplication.h>
#include <qmainwindow.h>
#include <qmessagebox.h>

#include <kapplication.h>
#include <kmainwindow.h>
#include <kcmdlineargs.h>
#include <kaboutdata.h>
#include <klocale.h>


#include <gwenhywfar/debug.h>

#include <stdio.h>

#include "accountlist.h"
#include "kbanking.h"
#include <aqbanking/banking.h>
#include "settings.h"
#include "inputbox.h"
#include "jobview.h"



#define PRG_VERSION "0.1"


static const char *description =
    I18N_NOOP("Test for KBanking");


static KCmdLineOptions options[] ={
    { 0, 0, 0 }
    // INSERT YOUR COMMANDLINE OPTIONS HERE
};




int test1(int argc, char **argv) {
  KApplication *a;
  KBankingSettings *bs;
  KBanking *kb;
  int result;

  KAboutData aboutData("kbanking", I18N_NOOP("KBanking"),
                       PRG_VERSION,
                       description,
                       KAboutData::License_GPL,
                       "(c) 2004, Martin Preuss",
                       0,
                       0,
                       "aqmoney@aquamaniac.de");

  aboutData.addAuthor("Martin Preuss",0, "aqmoney@aquamaniac.de");
  KCmdLineArgs::init( argc, argv, &aboutData );
  KCmdLineArgs::addCmdLineOptions( options ); // Add our own options.

  GWEN_Logger_SetLevel(0, GWEN_LoggerLevelInfo);
  GWEN_Logger_SetLevel("aqhbci", GWEN_LoggerLevelInfo); // DEBUG
  GWEN_Logger_SetLevel(AQBANKING_LOGDOMAIN, GWEN_LoggerLevelInfo);

  kb=new KBanking("kbanking-test");
  if (kb->init()) {
    fprintf(stderr, "Error on init.\n");
    return 2;
  }

  a=new KApplication();

  bs=new KBankingSettings(kb, 0, "BankingSettings");
  if (bs->init()) {
    fprintf(stderr, "Error on init of settings dialog.\n");
    return 3;
  }

  a->connect(a, SIGNAL(lastWindowClosed()), a, SLOT(quit()));
  result=bs->exec();

  if (bs->fini()) {
    fprintf(stderr, "Error on fini of settings dialog.\n");
  }

  if (kb->fini()) {
    QMessageBox::critical(0,
                          QWidget::tr("Deinitialization Error"),
                          QWidget::tr("<qt>"
                                      "Could not deinitialize KBanking."
                                      "</qt>"
                                     ),
                          QWidget::tr("Dismiss"));
    return 3;
  }

  delete bs;
  delete kb;

  delete a;
  return result;
}



int test2(int argc, char **argv) {
  KApplication *a;
  KBanking *kb;
  JobView *jv;
  int result;

  KAboutData aboutData("kbanking", I18N_NOOP("KBanking"),
                       PRG_VERSION,
                       description,
                       KAboutData::License_GPL,
                       "(c) 2004, Martin Preuss",
                       0,
                       0,
                       "aqmoney@aquamaniac.de");

  aboutData.addAuthor("Martin Preuss",0, "aqmoney@aquamaniac.de");
  KCmdLineArgs::init( argc, argv, &aboutData );
  KCmdLineArgs::addCmdLineOptions( options ); // Add our own options.

  GWEN_Logger_SetLevel(0, GWEN_LoggerLevelInfo);
  GWEN_Logger_SetLevel("aqhbci", GWEN_LoggerLevelInfo); // DEBUG
  GWEN_Logger_SetLevel(AQBANKING_LOGDOMAIN, GWEN_LoggerLevelInfo);

  kb=new KBanking("kbanking-test");
  if (kb->init()) {
    fprintf(stderr, "Error on init.\n");
    return 2;
  }

  a=new KApplication();

  jv=new JobView(kb, 0, "JobView");
  if (!jv->init()) {
    fprintf(stderr, "Error on init of settings dialog.\n");
    return 3;
  }

  a->connect(a, SIGNAL(lastWindowClosed()), a, SLOT(quit()));
  jv->show();
  result=a->exec();

  if (!jv->fini()) {
    fprintf(stderr, "Error on fini of settings dialog.\n");
  }

  if (kb->fini()) {
    QMessageBox::critical(0,
                          QWidget::tr("Deinitialization Error"),
                          QWidget::tr("<qt>"
                                      "Could not deinitialize KBanking."
                                      "</qt>"
                                     ),
                          QWidget::tr("Dismiss"));
    return 3;
  }

  delete jv;
  delete kb;

  return result;
}



int test3(int argc, char **argv) {
  KApplication *a;
  KBanking *kb;
  int result;
  int rv;

  KAboutData aboutData("kbanking", I18N_NOOP("KBanking"),
                       PRG_VERSION,
                       description,
                       KAboutData::License_GPL,
                       "(c) 2004, Martin Preuss",
                       0,
                       0,
                       "aqmoney@aquamaniac.de");

  aboutData.addAuthor("Martin Preuss",0, "aqmoney@aquamaniac.de");
  KCmdLineArgs::init( argc, argv, &aboutData );
  KCmdLineArgs::addCmdLineOptions( options ); // Add our own options.

  GWEN_Logger_SetLevel(0, GWEN_LoggerLevelInfo);
  GWEN_Logger_SetLevel("aqhbci", GWEN_LoggerLevelInfo); // DEBUG
  GWEN_Logger_SetLevel(AQBANKING_LOGDOMAIN, GWEN_LoggerLevelInfo);

  a=new KApplication();

  kb=new KBanking("kbanking-test");
  if (kb->init()) {
    fprintf(stderr, "Error on init.\n");
    return 2;
  }

  a->connect(a, SIGNAL(lastWindowClosed()), a, SLOT(quit()));
  rv=AB_Banking_MessageBox(kb->getCInterface(),
                           AB_BANKING_MSG_FLAGS_TYPE_WARN |
                           AB_BANKING_MSG_FLAGS_SEVERITY_DANGEROUS |
                           AB_BANKING_MSG_FLAGS_CONFIRM_B1,
                           "Insert Medium",
                           "Test message\n"
                           "<html>"
                           "This is a <b>test</b> message.<br>"
                           "</html>",
                           "OK", "Abort", 0);
  DBG_NOTICE(0, "Response was %d", rv);
  if (kb->fini()) {
    QMessageBox::critical(0,
                          QWidget::tr("Deinitialization Error"),
                          QWidget::tr("<qt>"
                                      "Could not deinitialize KBanking."
                                      "</qt>"
                                     ),
                          QWidget::tr("Dismiss"));
    return 3;
  }

  delete kb;

  return result;
}




int main(int argc, char **argv) {
  return test3(argc, argv);
}





